# shellcheck shell=bats
#
# Copyright © 2023 – 2024 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# bats file_tags=runtime-environment

load 'libs/bats-support/load'
load 'libs/bats-assert/load'
load 'libs/helpers'

setup_file() {
  bats_require_minimum_version 1.10.0
  _setup_environment
  cleanup_all
  pushd "$HOME" || return 1
  create_default_container
}

teardown_file() {
  popd || return 1
  cleanup_all
}

# bats test_tags=arch-fedora
@test "ulimit: real-time non-blocking time (hard)" {
  local limit
  limit=$(ulimit -H -R)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -R

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: real-time non-blocking time (soft)" {
  local limit
  limit=$(ulimit -S -R)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -R

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: core file size (hard)" {
  local limit
  limit=$(ulimit -H -c)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -c

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: core file size (soft)" {
  local limit
  limit=$(ulimit -S -c)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -c

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: data segment size (hard)" {
  local limit
  limit=$(ulimit -H -d)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -d

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: data segment size (soft)" {
  local limit
  limit=$(ulimit -S -d)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -d

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: scheduling priority (hard)" {
  local limit
  limit=$(ulimit -H -e)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -e

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: scheduling priority (soft)" {
  local limit
  limit=$(ulimit -S -e)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -e

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: file size (hard)" {
  local limit
  limit=$(ulimit -H -f)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -f

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: file size (soft)" {
  local limit
  limit=$(ulimit -S -f)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -f

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: number of pending signals (hard)" {
  local limit
  limit=$(ulimit -H -i)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -i

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: number of pending signals (soft)" {
  local limit
  limit=$(ulimit -S -i)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -i

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: locked memory size (hard)" {
  local limit
  limit=$(ulimit -H -l)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -l

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: locked memory size (soft)" {
  local limit
  limit=$(ulimit -S -l)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -l

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: resident memory size (hard)" {
  local limit
  limit=$(ulimit -H -m)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -m

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: resident memory size (soft)" {
  local limit
  limit=$(ulimit -S -m)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -m

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: number of open files (hard)" {
  local limit
  limit=$(ulimit -H -n)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -n

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: number of open files (soft)" {
  local limit
  limit=$(ulimit -H -n)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -n

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: pipe size (hard)" {
  local limit
  limit=$(ulimit -H -p)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -p

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: pipe size (soft)" {
  local limit
  limit=$(ulimit -S -p)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -p

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: POSIX message queue size (hard)" {
  local limit
  limit=$(ulimit -H -q)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -q

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: POSIX message queue size (soft)" {
  local limit
  limit=$(ulimit -S -q)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -q

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: real-time scheduling priority (hard)" {
  local limit
  limit=$(ulimit -H -r)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -r

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: real-time scheduling priority (soft)" {
  local limit
  limit=$(ulimit -S -r)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -r

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: stack size (hard)" {
  local limit
  limit=$(ulimit -H -s)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -s

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: stack size (soft)" {
  local limit
  limit=$(ulimit -S -s)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -s

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: CPU time (hard)" {
  local limit
  limit=$(ulimit -H -t)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -t

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: CPU time (soft)" {
  local limit
  limit=$(ulimit -S -t)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -t

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: number of user processes (hard)" {
  local limit
  limit=$(ulimit -H -u)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -u

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: number of user processes (soft)" {
  local limit
  limit=$(ulimit -S -u)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -u

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: virtual memory size (hard)" {
  local limit
  limit=$(ulimit -H -v)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -v

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: virtual memory size (soft)" {
  local limit
  limit=$(ulimit -S -v)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -v

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: number of file locks (hard)" {
  local limit
  limit=$(ulimit -H -x)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -H -x

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}

# bats test_tags=arch-fedora
@test "ulimit: number of file locks (soft)" {
  local limit
  limit=$(ulimit -S -x)

  run --keep-empty-lines --separate-stderr "$TOOLBX" run ulimit -S -x

  assert_success
  assert_line --index 0 "$limit"
  assert [ ${#lines[@]} -eq 1 ]

  # shellcheck disable=SC2154
  assert [ ${#stderr_lines[@]} -eq 0 ]
}
